package com.cgi.cihr;


import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.JFrame;

import com.cgi.cihr.gui.EISColourPane;
import com.cgi.cihr.gui.EISConnectDialog;
import com.cgi.cihr.gui.EISFileDialog;
import com.cgi.tools.ColumnReader;
import com.opentext.api.LAPI_ATTRIBUTES;
import com.opentext.api.LAPI_ATTRSERVER;
import com.opentext.api.LAPI_DOCUMENTS;
import com.opentext.api.LLIOException;
import com.opentext.api.LLSession;
import com.opentext.api.LLValue;
import com.opentext.api.PhysObj;
import com.opentext.api.RecMan;

/**
 * Test class to import EIS files into Livelink.
 * 
 * @author tomas.hansson
 *
 */
public class EISImportAction extends AbstractAction {
	private JFrame parent;
	private EISColourPane colourPane;
	private Properties props;
	private LLSession session;
	private LAPI_DOCUMENTS documents;
	private LAPI_ATTRIBUTES attributes;
	private RecMan recMan;
	private PhysObj physObj;
	
	/**
	 * Constructor.
	 * 
	 * @param cp
	 * @param aParent
	 */
	public EISImportAction(JFrame aParent, EISColourPane cp) {
		super("Import");
		this.parent = aParent;
		this.colourPane = cp;
	}
		
	public void actionPerformed(ActionEvent ev) {
		// Read properties
		try {
			props = new Properties();
			props.load(ClassLoader.getSystemResourceAsStream("eis.properties"));
		} catch (IOException e) {
			colourPane.writeException(e);
		}
		
		// Get login info
		EISConnectDialog cd = new EISConnectDialog(parent);
		cd.setVisible(true);
		
		if (cd.getUserName() == null || cd.getPassword() == null) {
			colourPane.writelnError("Import canceled.");
			return;
		}
		
		parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		// Establish connection to Livelink
		int status = 0;
		LLValue config = new LLValue().setAssoc();
		config.add("encoding", "UTF-8");
		session = new LLSession(props.getProperty("livelinkHost"), 
				Integer.parseInt(props.getProperty("livelinkPort")), 
				"", cd.getUserName(), cd.getPassword(), config);
		documents = new LAPI_DOCUMENTS(session);
		attributes = new LAPI_ATTRIBUTES(session);
		recMan = new RecMan(session);
		physObj = new PhysObj(session);
		
		DateFormat df = DateFormat.getDateTimeInstance();
		
		// Attempt to get to the Enterprise Workspace
		try {
			status = documents.AccessEnterpriseWS(new LLValue());
		
			if (status == 0) {
				colourPane.writelnInfo("Connected");
				colourPane.writelnInfo("Start of import: " + df.format(new Date()));
			} else {
				parent.setCursor(Cursor.getDefaultCursor());
				writeLAPIError("Unable to connect to Livelink.");
				return;
			}
		} catch (LLIOException e) {
			parent.setCursor(Cursor.getDefaultCursor());
			writeLAPIError("Unable to connect to Livelink.");
			return;
		}
		
		// Get the file to process.
		EISFileDialog fd = new EISFileDialog();
		File compFile = fd.getFile(parent);
		
		if (compFile == null) {
			colourPane.writelnError("Import canceled.");
			return;			
		}
		
		parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		// Read columns in EIS file into a list
		List<String[]> recList = readRecs(compFile);
		
		if (recList == null || recList.size() == 0) {
			parent.setCursor(Cursor.getDefaultCursor());
			colourPane.writelnError("Invalid file format.");
			return;
		}
		
		LLValue classInfo = getClassification();
		
		if (classInfo == null) {
			parent.setCursor(Cursor.getDefaultCursor());
			writeLAPIError("Failed to retrieve classification.");
			return;
		}
		
		// Get the category version
		LLValue catVersion = getCategoryVersion();
		
		if (catVersion == null) {
			parent.setCursor(Cursor.getDefaultCursor());
			writeLAPIError("Failed to retrieve category version.");
		}
		
		// Create the entries
		for (int i = 0; i < recList.size(); i++) {
			colourPane.writeInfo("Line: " + (i + 1) + " - ");
			LLValue recInfo = addRecord(catVersion, recList.get(i));

			if (recInfo == null) {
				writeLAPIError("Failed to add record " + (i + 1) + ".");
			}
		}
		
		colourPane.writelnInfo("End of import: " + df.format(new Date()));
		
		parent.setCursor(Cursor.getDefaultCursor());
	}
	
	// Method to read the records from EIS Import file
	private List<String[]> readRecs(File compFile) {
		String[] colStrings = props.getProperty("colWidths").split(", ");
		int[] colWidths = new int[colStrings.length];
		
		for (int i = 0; i < colStrings.length; i++) {
			colWidths[i] = Integer.parseInt(colStrings[i]);
		}
		
		List<String[]> colList = new ArrayList<String[]>();
		
		try {
			ColumnReader r = new ColumnReader(colWidths, compFile);
			
			String[] cols = r.readLines();
			
			while (cols != null) {
				colList.add(cols);
				cols = r.readLines();
			}
			
			r.close();
		} catch (IOException e) {
			colourPane.writeException(e);
			return null;
		}
		
		return colList;
	}
	
	/* Get (or create) the Competition Management folder as well as the sub-folders.
	    Add the classification and security info to the last folder
		The structure is as follows:
		Competition Management
		  YYYY
		    YYYYMM
		      YYYYMMXXX
		        First Initial of Last Name
	*/
	private LLValue createFolder(String compCode, String name) {
	    // Access the Enterprise Workspace
		LLValue wsInfo = new LLValue();
		LLValue retFolder = null;
		
	    int status = documents.AccessEnterpriseWS(wsInfo);
	    
	    if (status != 0) {
	    	writeLAPIError("Error accessing Enterprise WS");
	    	return null;
	    }
	    
	    int parentVol = wsInfo.toInteger("VolumeID");
	    int parentId = wsInfo.toInteger("ID");
	    
	    // Create the path string and create a folder structure
	    String path = props.getProperty("cmFolder") + ":" + compCode.substring(0, 4) + ":" + 
	    	compCode.substring(0, 6) + ":" + compCode + ":" + name.substring(0, 1);
	    String folders[] = path.split(":");
	    
	    for (int i = 0; i < folders.length; i++) {
	    	String folder = folders[i];
	    	LLValue folderInfo = null;
	    	LLValue children = new LLValue();
	    	
	    	status = documents.ListObjects(parentVol, parentId,
	        		null, null, LAPI_DOCUMENTS.PERM_CREATENODE, children);
	    	
	    	for (int j = 0; j < children.size(); j++) {
	    		LLValue info = new LLValue();        		
	        	status = documents.GetObjectInfo(children.toInteger(j, "VolumeID"), 
	        			children.toInteger(j, "ID"), info);
	        	
	    	    if (status != 0) {
	    	    	writeLAPIError("Unable to get children for folder " + children.toInteger(j, "ID"));
	    	    	return null;
	    	    }
	        	
	        	if (info.toString("Name").equalsIgnoreCase(folder)) {
	        		folderInfo = info;
	        		break;
	        	}
	    	}
	    	
	    	if (folderInfo == null) {
	    		folderInfo = new LLValue();
	        	status = documents.CreateObjectEx(parentVol, parentId,
	        			LAPI_DOCUMENTS.OBJECTTYPE, LAPI_DOCUMENTS.FOLDERSUBTYPE, folder, 
	        			new LLValue().setAssocNotSet(), folderInfo);
	        	
	    	    if (status != 0) {
	    	    	writeLAPIError("Unable to create folder " + folder);
	    	    	return null;
	    	    }
	    	}
	
	    	parentVol = folderInfo.toInteger("VolumeID");
	    	parentId = folderInfo.toInteger("ID");

	    	// Last folder
	    	if (i == (folders.length - 1)) {
	    		retFolder = folderInfo;
	    		
	    		// Add classification
	    		LLValue classification = getClassification();
	    		
	    		if (classification != null) {
		        	status = recMan.RMClassify(parentId, classification.toInteger("NodeID"), 
		        			new LLValue().setAssocNotSet());
	
		        	if (status != 0) {
		        		writeLAPIError("Unable to classify object " + parentId);
		        	}
	    		}
	        	
	    		/* REMOVED - Sec Clrnc will be done through inheritance
	        	// Add security level
	            LLValue secInfo = new LLValue().setAssoc();
	            LLValue secStatus = new LLValue();
	            secInfo.add("secLevel", Integer.parseInt(props.getProperty("securityLevel")));
	            
	            status = secMan.RMSecSetItemSec(parentId, secInfo, secStatus);
	            
	            if (status != 0) {
	            	writeLAPIError("Unable to set security on object " + parentId);
	            }
	            */	    		
	    	}
	    }
	    
	    return retFolder;
	}

	// Get the category to use
	private LLValue getCategoryVersion() {
        int id = -1;
        
        LLValue catWs = new LLValue();
        int status = documents.AccessCategoryWS(catWs);
        
        if (status != 0) {
        	return null;
        }
        
        LLValue children = new LLValue();       
        status = documents.ListObjects(catWs.toInteger("VolumeID"), catWs.toInteger("ID"), null, null, 
        		LAPI_DOCUMENTS.PERM_SEE, children);

        if (status != 0) {
        	return null;
        }

        for (int i = 0; i < children.size(); i++) {
        	LLValue info = new LLValue();
        	status = documents.GetObjectInfo(children.toInteger(i, "VolumeID"), 
        			children.toInteger(i, "ID"), info);
        	
            if (status != 0) {
            	return null;
            }

        	if (info.toString("Name").equalsIgnoreCase(props.getProperty("categoryName"))) {
        		id = info.toInteger("ID");
        		break;
        	}
        }
        
        if (id < 0) {
        	return null;
        }
        
        LLValue catID = new LLValue().setAssoc();
        LLValue catVersion = new LLValue().setAssoc();
        
        catID.add("Type", LAPI_ATTRSERVER.CATEGORY_TYPE_LIBRARY);
        catID.add("ID", id);
        status = documents.FetchCategoryVersion(catID, catVersion);
        
        if (status != 0) {
        	return null;
        }

        return catVersion;
	}
	
	// Get the classification to use
	private LLValue getClassification() {
		int status = 0;
		LLValue classInfo = null;
		LLValue classRoot = new LLValue();
		LLValue children = new LLValue();

        status = recMan.RMGetClassificationRoot(classRoot);
        
        if (status != 0) {
        	writeLAPIError("Unable to get classification root");
        	return null;
        }
        
        status = recMan.RMGetClassificationChildren(classRoot.toInteger("NodeID"), children);

        if (status != 0) {
        	writeLAPIError("Unable to get classification children");        	
        	return null;
        }
        
        for (int i = 0; i < children.size(); i++) {
        	if (children.toString(i, "FileNumber").equalsIgnoreCase(props.getProperty("classificationNumber"))) {
        		classInfo = children.toValue(i);
        	}
        }
				
		return classInfo;
	}
	
	// Add the actual physical object
	private LLValue addRecord(LLValue catVersion, String[] rec) {
		int status = 0;

		// Get the folder
		LLValue folderInfo = createFolder(rec[3], rec[0]);
		
		if (folderInfo == null) {
			return null;
		}
		
		LLValue objectInfo = new LLValue();
		LLValue cats = new LLValue().setList();
		LLValue createInfo = new LLValue().setAssoc();
		LLValue request = new LLValue().setAssoc();
		LLValue sysAttrs = new LLValue().setAssoc();
		
		// Physical objects info
		request.add("selectedMedia", Integer.parseInt(props.getProperty("physicalVolumeId")));
		request.add("poLocation", "CIHR 1");
		request.add("poNumCopies", 1);
		createInfo.add("request", request);
		
		// System attribute
		sysAttrs.add("Language", "Bilingual");		
		createInfo.add("requiredAttributes", sysAttrs);
				
		if (catVersion != null) {
	       	LLValue val = new LLValue().setList();
	        val.add(rec[3]);
	        status = attributes.AttrSetValues(catVersion, "Competition",
	        		LAPI_ATTRSERVER.ATTR_DATAVALUES, null, val);
	        
	        val = new LLValue().setList();
	        val.add(rec[0]);
	        status = attributes.AttrSetValues(catVersion, "Nominated Principle Investigator",
	        		LAPI_ATTRSERVER.ATTR_DATAVALUES, null, val);
	        
	        val = new LLValue().setList();
	        val.add(rec[1]);
	        status = attributes.AttrSetValues(catVersion, "PIN Number",
	        		LAPI_ATTRSERVER.ATTR_DATAVALUES, null, val);
	        
	        val = new LLValue().setList();
	        val.add(rec[2]);
	        status = attributes.AttrSetValues(catVersion, "Application Number",
	        		LAPI_ATTRSERVER.ATTR_DATAVALUES, null, val);
	        
	        val = new LLValue().setList();
	        val.add(rec[4]);
	        status = attributes.AttrSetValues(catVersion, "Project Title",
	        		LAPI_ATTRSERVER.ATTR_DATAVALUES, null, val);

	        cats.add(catVersion);
	        createInfo.add("Categories", cats);
		}
		
		status = documents.CreateObjectEx(folderInfo.toInteger("VolumeID"), 
				folderInfo.toInteger("ID"), LAPI_DOCUMENTS.OBJECTTYPE, 412,
				rec[0] + " - " + rec[2], createInfo, objectInfo);
		
		if (status != 0) {
			return null;
		}
		
		colourPane.writelnInfo("Created volume: " + objectInfo.toInteger("ID"));
		
		// Label generation
		status = physObj.PhysObjGenerateLabel(objectInfo.toInteger("ID"), 
				props.getProperty("labelType"), 1, new LLValue(false), "", "", "");
		
		if (status != 0) {
			return null;
		}
		
		return objectInfo;
	}
	
	private void writeLAPIError(String msg) {
		colourPane.writelnError(msg);
		colourPane.writelnError(session.getStatusMessage());
		colourPane.writelnError(session.getErrMsg());
		colourPane.writelnError(session.getApiError());		
	}
}
